/**
 * Windows 96 Wiki (Wiki96). Copyright (C) Windows 96 Team 2020.
 */

(function() {
    // Functions
    function dlgAlert(msg) {
        if(typeof parent.w96 === 'object') {
            // Use w96 dialog
            const m = parent.w96.ui.MsgBoxSimple.info("Wiki96", msg, "OK");
            m.setSize(392, 115);
            m.dlg.center();
        } else {
            alert(msg);
        }
    }

    // Config
    const WIKI_JSON_PATH = "./data/wiki.json";
    const WIKI_PAGES_DIR = "./pages";
    const HOME_PAGE = "home";
    
    // Session object
    let session = {
        wikiObject: {}
    }

    function pageExists(pageId) {
        return session.wikiObject.pages.find((p)=>p.id == pageId);
    }

    async function loadPage(pageId) {
        const pageContainer = document.querySelector(".main>.content>.page");
        const pageTitleContainer = document.querySelector(".main>header>h3");

        if(!pageExists(pageId)) {
            // Display not found
            pageTitleContainer.innerText = "Page Not Found";
            const req_404 = await fetch(WIKI_PAGES_DIR + "/__essential__/404.html");
            pageContainer.innerHTML = await req_404.text();
            return;
        }

        const page = session.wikiObject.pages.find((p)=>p.id == pageId);
        pageTitleContainer.innerText = page.title;
        
        const pageReq = await fetch(WIKI_PAGES_DIR + "/" + page.src);
        const htmlText = await pageReq.text();
        pageContainer.innerHTML = htmlText;
        pageContainer.querySelectorAll("script[exec]").forEach((el)=>{
            try {
                eval(el.innerText);
            } catch(e) {
                console.error(e);
            }
        });

        window.location.hash = pageId;
    }

    async function randomPage() {
        const randomPage = wiki96.session.wikiObject.pages[Math.floor(Math.random() * wiki96.session.wikiObject.pages.length)];
        return await loadPage(randomPage.id);
    }

    // Main function
    async function app() {
        // Retrieve Wiki JSON (contains all sections and pages)
        const wikiJsonResponse = await fetch(WIKI_JSON_PATH);
        session.wikiObject = await wikiJsonResponse.json();

        // Create sections
        const sectionsContainer = document.querySelector(".main>.content>.sections");

        session.wikiObject.sections.forEach((section)=>{
            // Create section DOM element
            let sectionDomElement = document.createElement('div');
            sectionDomElement.classList.add('section');

            let sectionTitleEl = document.createElement('div');
            sectionTitleEl.classList.add('title');
            sectionTitleEl.innerText = section.name;
            sectionDomElement.appendChild(sectionTitleEl);

            // Add links
            section.links.forEach((link)=>{
                let linkEl = document.createElement('a');
                linkEl.classList.add('link');
                linkEl.innerText = link.caption;

                if(link.href.startsWith("$")) { // Set event onclick for loading page
                    linkEl.href = "#" + link.href.substring(1);

                    linkEl.onclick = ()=>{
                        loadPage(link.href.substring(1));
                    }
                } else // Set href to link
                    linkEl.href = link.href;

                sectionDomElement.appendChild(linkEl);
            });

            sectionsContainer.appendChild(sectionDomElement);
        });

        // Assign button events
        const permalinkButton = document.querySelector('.permalink');
        permalinkButton.onclick = ()=>{
            dlgAlert("Article Permalink: " + window.location.href);
        }

        const contributeButton = document.querySelector('.contribute');
        contributeButton.onclick = ()=>{
            window.open("https://git.sys36.net/windows-96/wiki96");
        }

        // Expose Wiki
        window.wiki96 = {
            loadPage,
            pageExists,
            randomPage,
            session
        }

        // Load page from hash. If hash is empty, load default page.
        if(window.location.hash.trim() != "") {
            await loadPage(window.location.hash.substring(1));
        } else {
            await loadPage(HOME_PAGE);
        }
    }

    window._app = app;
})();